@@ -1,7 +1,7 @@
 /**
  * MIT License
  * 
- * Copyright (c) 2017 Tessil
+ * Copyright (c) 2017 Thibaut Goetghebuer-Planchon <tessil@gmx.com>
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -68,11 +68,12 @@
 #if (defined(__cpp_exceptions) || defined(__EXCEPTIONS) || (defined (_MSC_VER) && defined (_CPPUNWIND))) && !defined(TSL_NO_EXCEPTIONS)
 #    define TSL_OH_THROW_OR_TERMINATE(ex, msg) throw ex(msg)
 #else
+#    define TSL_OH_NO_EXCEPTIONS
 #    ifdef NDEBUG
 #        define TSL_OH_THROW_OR_TERMINATE(ex, msg) std::terminate()
 #    else
-#        include <cstdio>
-#        define TSL_OH_THROW_OR_TERMINATE(ex, msg) do { std::fprintf(stderr, msg); std::terminate(); } while(0)
+#        include <iostream>
+#        define TSL_OH_THROW_OR_TERMINATE(ex, msg) do { std::cerr << msg << std::endl; std::terminate(); } while(0)
 #    endif
 #endif
 
@@ -105,6 +106,47 @@ struct is_vector<T, typename std::enable_if<
                     >::type>: std::true_type {
 };
 
+// Only available in C++17, we need to be compatible with C++11
+template<class T>
+const T& clamp( const T& v, const T& lo, const T& hi) {
+    return std::min(hi, std::max(lo, v));
+}
+
+template<typename T, typename U>
+static T numeric_cast(U value, const char* error_message = "numeric_cast() failed.") {
+    T ret = static_cast<T>(value);
+    if(static_cast<U>(ret) != value) {
+        TSL_OH_THROW_OR_TERMINATE(std::runtime_error, error_message);
+    }
+    
+    const bool is_same_signedness = (std::is_unsigned<T>::value && std::is_unsigned<U>::value) ||
+                                    (std::is_signed<T>::value && std::is_signed<U>::value);
+    if(!is_same_signedness && (ret < T{}) != (value < U{})) {
+        TSL_OH_THROW_OR_TERMINATE(std::runtime_error, error_message);
+    }
+    
+    return ret;
+}
+
+
+/**
+ * Fixed size type used to represent size_type values on serialization. Need to be big enough
+ * to represent a std::size_t on 32 and 64 bits platforms, and must be the same size on both platforms.
+ */
+using slz_size_type = std::uint64_t;
+static_assert(std::numeric_limits<slz_size_type>::max() >= std::numeric_limits<std::size_t>::max(),
+              "slz_size_type must be >= std::size_t");
+
+template<class T, class Deserializer>
+static T deserialize_value(Deserializer& deserializer) {
+    // MSVC < 2017 is not conformant, circumvent the problem by removing the template keyword
+#if defined (_MSC_VER) && _MSC_VER < 1910
+    return deserializer.Deserializer::operator()<T>();
+#else
+    return deserializer.Deserializer::template operator()<T>();
+#endif
+}
+
 
 /**
  * Each bucket entry stores an index which is the index in m_values corresponding to the bucket's value 
@@ -167,6 +209,27 @@ public:
         m_hash = truncate_hash(hash);
     }
     
+    template<class Serializer>
+    void serialize(Serializer& serializer) const {
+        const slz_size_type index = m_index;
+        serializer(index);
+        
+        const slz_size_type hash = m_hash;
+        serializer(hash);
+    }
+    
+    template<class Deserializer>
+    static bucket_entry deserialize(Deserializer& deserializer) {
+        const slz_size_type index = deserialize_value<slz_size_type>(deserializer);
+        const slz_size_type hash = deserialize_value<slz_size_type>(deserializer);
+        
+        bucket_entry bentry;
+        bentry.m_index = numeric_cast<index_type>(index, "Deserialized index is too big.");
+        bentry.m_hash = numeric_cast<truncated_hash_type>(hash, "Deserialized hash is too big.");
+        
+        return bentry;
+    }
+    
     
     
     static truncated_hash_type truncate_hash(std::size_t hash) noexcept {
@@ -202,7 +265,7 @@ private:
  * 
  * 
  * 
- * The orderd_hash structure is a hash table which preserves the order of insertion of the elements.
+ * The ordered_hash structure is a hash table which preserves the order of insertion of the elements.
  * To do so, it stores the values in the ValueTypeContainer (m_values) using emplace_back at each
  * insertion of a new element. Another structure (m_buckets of type std::vector<bucket_entry>) will 
  * serve as buckets array for the hash table part. Each bucket stores an index which corresponds to 
@@ -384,12 +447,12 @@ public:
                                          KeyEqual(equal), 
                                          m_buckets_data(alloc), 
                                          m_buckets(static_empty_bucket_ptr()), 
-                                         m_mask(0),
+                                         m_hash_mask(0),
                                          m_values(alloc), 
                                          m_grow_on_next_insert(false)
     {
         if(bucket_count > max_bucket_count()) {
-            TSL_OH_THROW_OR_TERMINATE(std::length_error, "The map exceeds its maxmimum size.");
+            TSL_OH_THROW_OR_TERMINATE(std::length_error, "The map exceeds its maximum size.");
         }
         
         if(bucket_count > 0) {
@@ -397,7 +460,7 @@ public:
             
             m_buckets_data.resize(bucket_count);
             m_buckets = m_buckets_data.data(),
-            m_mask = bucket_count - 1; 
+            m_hash_mask = bucket_count - 1;
         }
         
         this->max_load_factor(max_load_factor);
@@ -408,11 +471,11 @@ public:
                                              m_buckets_data(other.m_buckets_data),
                                              m_buckets(m_buckets_data.empty()?static_empty_bucket_ptr():
                                                                               m_buckets_data.data()),
-                                             m_mask(other.m_mask),
+                                             m_hash_mask(other.m_hash_mask),
                                              m_values(other.m_values),
-                                             m_grow_on_next_insert(other.m_grow_on_next_insert),
+                                             m_load_threshold(other.m_load_threshold),
                                              m_max_load_factor(other.m_max_load_factor),
-                                             m_load_threshold(other.m_load_threshold)
+                                             m_grow_on_next_insert(other.m_grow_on_next_insert)
     {
     }
     
@@ -425,18 +488,18 @@ public:
                                             m_buckets_data(std::move(other.m_buckets_data)),
                                             m_buckets(m_buckets_data.empty()?static_empty_bucket_ptr():
                                                                              m_buckets_data.data()),
-                                            m_mask(other.m_mask),
+                                            m_hash_mask(other.m_hash_mask),
                                             m_values(std::move(other.m_values)),
-                                            m_grow_on_next_insert(other.m_grow_on_next_insert),
+                                            m_load_threshold(other.m_load_threshold),
                                             m_max_load_factor(other.m_max_load_factor),
-                                            m_load_threshold(other.m_load_threshold)
+                                            m_grow_on_next_insert(other.m_grow_on_next_insert)
     {
         other.m_buckets_data.clear();
         other.m_buckets = static_empty_bucket_ptr();
-        other.m_mask = 0;
+        other.m_hash_mask = 0;
         other.m_values.clear();
-        other.m_grow_on_next_insert = false;
         other.m_load_threshold = 0;
+        other.m_grow_on_next_insert = false;
     }
     
     ordered_hash& operator=(const ordered_hash& other) {
@@ -448,11 +511,11 @@ public:
             m_buckets = m_buckets_data.empty()?static_empty_bucket_ptr():
                                                m_buckets_data.data();
                                                         
-            m_mask = other.m_mask;
+            m_hash_mask = other.m_hash_mask;
             m_values = other.m_values;
-            m_grow_on_next_insert = other.m_grow_on_next_insert;
-            m_max_load_factor = other.m_max_load_factor;
             m_load_threshold = other.m_load_threshold;
+            m_max_load_factor = other.m_max_load_factor;
+            m_grow_on_next_insert = other.m_grow_on_next_insert;
         }
         
         return *this;
@@ -728,11 +791,11 @@ public:
         swap(static_cast<KeyEqual&>(*this), static_cast<KeyEqual&>(other));
         swap(m_buckets_data, other.m_buckets_data);
         swap(m_buckets, other.m_buckets);
-        swap(m_mask, other.m_mask);
+        swap(m_hash_mask, other.m_hash_mask);
         swap(m_values, other.m_values);
-        swap(m_grow_on_next_insert, other.m_grow_on_next_insert);
-        swap(m_max_load_factor, other.m_max_load_factor);
         swap(m_load_threshold, other.m_load_threshold);
+        swap(m_max_load_factor, other.m_max_load_factor);
+        swap(m_grow_on_next_insert, other.m_grow_on_next_insert);
     }
     
         
@@ -811,6 +874,17 @@ public:
         return (it_bucket != m_buckets_data.cend())?const_iterator(m_values.begin() + it_bucket->index()):end();
     }
     
+
+    template<class K>
+    bool contains(const K& key) const {
+        return contains(key, hash_key(key));
+    }
+    
+    template<class K>
+    bool contains(const K& key, std::size_t hash) const {
+        return find(key, hash) != cend();
+    }
+    
     
     template<class K>
     std::pair<iterator, iterator> equal_range(const K& key) {
@@ -862,7 +936,10 @@ public:
     }
     
     void max_load_factor(float ml) {
-        m_max_load_factor = std::max(0.1f, std::min(ml, 0.95f));
+        m_max_load_factor = clamp(ml, float(MAX_LOAD_FACTOR__MINIMUM), 
+                                      float(MAX_LOAD_FACTOR__MAXIMUM));
+
+        m_max_load_factor = ml;
         m_load_threshold = size_type(float(bucket_count())*m_max_load_factor);
     }
     
@@ -1013,6 +1090,16 @@ public:
         return 1;
     }
     
+    template<class Serializer>
+    void serialize(Serializer& serializer) const {
+        serialize_impl(serializer);
+    }
+    
+    template<class Deserializer>
+    void deserialize(Deserializer& deserializer, bool hash_compatible) {
+        deserialize_impl(deserializer, hash_compatible);
+    }
+    
     friend bool operator==(const ordered_hash& lhs, const ordered_hash& rhs) {
         return lhs.m_values == rhs.m_values;
     }
@@ -1085,7 +1172,7 @@ private:
         tsl_oh_assert(bucket_count >= size_type(std::ceil(float(size())/max_load_factor())));
         
         if(bucket_count > max_bucket_count()) {
-            TSL_OH_THROW_OR_TERMINATE(std::length_error, "The map exceeds its maxmimum size.");
+            TSL_OH_THROW_OR_TERMINATE(std::length_error, "The map exceeds its maximum size.");
         }
         
         if(bucket_count > 0) {
@@ -1103,7 +1190,7 @@ private:
                                            m_buckets_data.data();
         // Everything should be noexcept from here.
         
-        m_mask = (bucket_count > 0)?(bucket_count - 1):0;
+        m_hash_mask = (bucket_count > 0)?(bucket_count - 1):0;
         this->max_load_factor(m_max_load_factor);
         m_grow_on_next_insert = false;
         
@@ -1363,7 +1450,7 @@ private:
     }
     
     std::size_t bucket_for_hash(std::size_t hash) const noexcept {
-        return hash & m_mask;
+        return hash & m_hash_mask;
     }    
     
     std::size_t iterator_to_index(const_iterator it) const noexcept {
@@ -1388,6 +1475,83 @@ private:
         }
     }
     
+    template<class Serializer>
+    void serialize_impl(Serializer& serializer) const {
+        const slz_size_type version = SERIALIZATION_PROTOCOL_VERSION;
+        serializer(version);
+        
+        const slz_size_type nb_elements = m_values.size();
+        serializer(nb_elements);
+        
+        const slz_size_type bucket_count = m_buckets_data.size();
+        serializer(bucket_count);
+        
+        const float max_load_factor = m_max_load_factor;
+        serializer(max_load_factor);
+        
+        
+        for(const value_type& value: m_values) {
+            serializer(value);
+        }
+        
+        for(const bucket_entry& bucket: m_buckets_data) {
+            bucket.serialize(serializer);
+        }
+    }
+    
+    template<class Deserializer>
+    void deserialize_impl(Deserializer& deserializer, bool hash_compatible) {
+        tsl_oh_assert(m_buckets_data.empty()); // Current hash table must be empty
+        
+        const slz_size_type version = deserialize_value<slz_size_type>(deserializer);
+        // For now we only have one version of the serialization protocol. 
+        // If it doesn't match there is a problem with the file.
+        if(version != SERIALIZATION_PROTOCOL_VERSION) {
+            TSL_OH_THROW_OR_TERMINATE(std::runtime_error, "Can't deserialize the ordered_map/set. "
+                                                          "The protocol version header is invalid.");
+        }
+        
+        const slz_size_type nb_elements = deserialize_value<slz_size_type>(deserializer);
+        const slz_size_type bucket_count_ds = deserialize_value<slz_size_type>(deserializer);
+        const float max_load_factor = deserialize_value<float>(deserializer);
+
+        if(max_load_factor < MAX_LOAD_FACTOR__MINIMUM || max_load_factor > MAX_LOAD_FACTOR__MAXIMUM) {
+            TSL_OH_THROW_OR_TERMINATE(std::runtime_error, "Invalid max_load_factor. Check that the serializer "
+                                                          "and deserializer support floats correctly as they "
+                                                          "can be converted implicitly to ints.");
+        }
+        
+        
+        this->max_load_factor(max_load_factor);
+        
+        if(bucket_count_ds == 0) {
+            tsl_oh_assert(nb_elements == 0);
+            return;
+        }
+        
+        
+        if(!hash_compatible) {
+            reserve(numeric_cast<size_type>(nb_elements, "Deserialized nb_elements is too big."));
+            for(slz_size_type el = 0; el < nb_elements; el++) {
+                insert(deserialize_value<value_type>(deserializer));
+            }
+        }
+        else {
+            m_buckets_data.reserve(numeric_cast<size_type>(bucket_count_ds, "Deserialized bucket_count is too big."));
+            m_buckets = m_buckets_data.data(),
+            m_hash_mask = m_buckets_data.capacity() - 1; 
+            
+            reserve_space_for_values(numeric_cast<size_type>(nb_elements, "Deserialized nb_elements is too big."));
+            for(slz_size_type el = 0; el < nb_elements; el++) {
+                m_values.push_back(deserialize_value<value_type>(deserializer));
+            }
+            
+            for(slz_size_type b = 0; b < bucket_count_ds; b++) {
+                m_buckets_data.push_back(bucket_entry::deserialize(deserializer));
+            }
+        }
+    }
+    
     static std::size_t round_up_to_power_of_two(std::size_t value) {
         if(is_power_of_two(value)) {
             return value;
@@ -1415,9 +1579,16 @@ public:
     static constexpr float DEFAULT_MAX_LOAD_FACTOR = 0.75f;
 
 private:    
+    static constexpr float MAX_LOAD_FACTOR__MINIMUM = 0.1f;
+    static constexpr float MAX_LOAD_FACTOR__MAXIMUM = 0.95f;
+
     static const size_type REHASH_ON_HIGH_NB_PROBES__NPROBES = 128;
     static constexpr float REHASH_ON_HIGH_NB_PROBES__MIN_LOAD_FACTOR = 0.15f;
     
+    /**
+     * Protocol version currenlty used for serialization.
+     */
+    static const slz_size_type SERIALIZATION_PROTOCOL_VERSION = 1;
     
     /**
      * Return an always valid pointer to an static empty bucket_entry with last_bucket() == true.
@@ -1439,13 +1610,14 @@ private:
      */
     bucket_entry* m_buckets;
     
-    size_type m_mask;
+    size_type m_hash_mask;
     
     values_container_type m_values;
     
-    bool m_grow_on_next_insert;
-    float m_max_load_factor;
     size_type m_load_threshold;
+    float m_max_load_factor;
+    
+    bool m_grow_on_next_insert;
 };
 
 
